// (C)2007 Attila Kovacs <attila@submm.caltech.edu>

import java.util.*;

class SourceBoxingThread extends Thread {
		public static Vector2D minOffset, maxOffset;
		
		Vector<LabocaScan[]> scans = new Vector<LabocaScan[]>();
		MiniCRUSH crush;
		
		public SourceBoxingThread(MiniCRUSH crush) { this.crush = crush; }
		
		public void addScan(LabocaScan[] scan) { 
			scans.add(scan); 
			for(int k=0; k<scan.length; k++) crush.queue.add(scan[k]);
		}
		
		public void run() {		
			double mindX = Double.POSITIVE_INFINITY, maxdX = Double.NEGATIVE_INFINITY;
			double mindY = Double.POSITIVE_INFINITY, maxdY = Double.NEGATIVE_INFINITY;
			
			for(LabocaScan[] scan : scans) for(int k=0; k<scan.length; k++) {
				
				LabocaScan subscan = scan[k];
				int[] channelSelection = subscan.allChannels;
				
				subscan.map = crush.map;
				
				for(int t=0; t<subscan.frames; t++) {
					LabocaFrame exposure = subscan.frame[t];
					if(exposure != null) if((exposure.flag & LabocaFrame.SKIP_SOURCE) == 0) {

						for(int q=0; q<channelSelection.length; q++) {
							Vector2D offset = subscan.getMapOffsets(t, channelSelection[q]);

							if(offset.x < mindX) mindX = offset.x;
							if(offset.x > maxdX) maxdX = offset.x;
							if(offset.y < mindY) mindY = offset.y;
							if(offset.y > maxdY) maxdY = offset.y;
							
						}
					}
				}
				
				updateCorners(mindX, mindY, maxdX, maxdY);
				crush.checkout(subscan);
			}
		}	
		
		public static synchronized void updateCorners(double mindX, double mindY, double maxdX, double maxdY) {
			if(minOffset == null) {
				minOffset = new Vector2D(mindX, mindY);
				maxOffset = new Vector2D(maxdX, maxdY);
			}
			else {
				minOffset.x = Math.min(mindX, minOffset.x);
				minOffset.y = Math.min(mindY, minOffset.y);
				maxOffset.x = Math.max(maxdX, maxOffset.x);
				maxOffset.y = Math.max(maxdY, maxOffset.y);
			}
		}
	}