// (C)2007 Attila Kovacs <attila@submm.caltech.edu>

import java.io.*;
import java.util.*;


public class TauInterpolator extends Vector<SkyDipEntry> {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7962217110619389946L;

	public TauInterpolator() {
		
		
	}
	
	public void readData(String fileName) throws IOException {
		BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
		String line = null;
		while((line = in.readLine()) != null) if(line.length() > 0) if(line.charAt(0) != '#') {
			StringTokenizer tokens = new StringTokenizer(line);
			SkyDipEntry skydip = new SkyDipEntry();
			skydip.scanNo = Integer.parseInt(tokens.nextToken());
			tokens.nextToken();
			skydip.MJD = Double.parseDouble(tokens.nextToken());
			skydip.tau = Double.parseDouble(tokens.nextToken());
			add(skydip);
		}
		
		// Order the list...
		Collections.sort(this);		
	}
	
	
	// Linear interpolation.
	// Throws Exception if MJD is outside of the interpolator range.
	public double getValue(double MJD) throws ArrayIndexOutOfBoundsException {
		int i = 0;
		while(get(i).MJD < MJD) i++;
		
		if(i == 0) throw new ArrayIndexOutOfBoundsException(i);
		
		double dt1 = MJD - get(i-1).MJD;
		double dt2 = get(i).MJD - MJD;
		
		return (dt2 * get(i-1).tau + dt1 * get(i).tau) / (dt1 + dt2);	
	}
	
	
}

